@@ -98,21 +98,22 @@ BYTE* Compressor::NextOutputRaster(COLORTYPE color)
 	}
 }
 
-Mode9::Mode9(SystemServices* pSys,unsigned int RasterSize)
+Mode9::Mode9 (SystemServices* pSys,unsigned int RasterSize, BOOL bVIPPrinter)
     : Compressor(pSys,RasterSize, TRUE)
 {
     if (constructor_error != NO_ERROR)  // if error in base constructor
         return;
 
-	// In the worst case, compression expands data by 50%
-	compressBuf = (BYTE*)pSS->AllocMem(RasterSize + RasterSize/2);
+	// Allocate double the RasterSize to accommodate worst case
+	compressBuf = (BYTE*)pSS->AllocMem(RasterSize * 2);
 	if (compressBuf == NULL)
 		constructor_error=ALLOCMEM_ERROR;
 
-	memset(compressBuf, 0, RasterSize + RasterSize/2);
+	memset(compressBuf, 0, RasterSize * 2);
     memset(SeedRow,0,RasterSize);
 
 	ResetSeedRow = FALSE;
+    m_bVIPPrinter = bVIPPrinter;
 }
 
 Mode9::~Mode9()
@@ -198,7 +199,7 @@ BOOL Mode9::Process(RASTERDATA* input)
 		return TRUE;
 	}
 
-	memset(compressBuf, 0, inputsize + inputsize/2);
+	memset(compressBuf, 0, inputsize * 2);
 
     unsigned int originalsize=input->rastersize[myplane];
 	unsigned int size=input->rastersize[myplane];
@@ -271,7 +272,13 @@ BOOL Mode9::Process(RASTERDATA* input)
     {
         offset = 0;
 
-        if (seeded)
+/*
+ *      If we are here because we are doing KRGB processing, don't offset off of
+ *      the seedrow because seedrow here and one setup with mode10 may be out of sync,
+ *      causing artifacts in the output.
+ */
+
+        if (seeded && !m_bVIPPrinter)
         {
             /* find a difference between the seed row and this row. */
             while ((*sptr++ == *nptr++) && (offset < size) )
