@@ -620,7 +620,7 @@ int __attribute__ ((visibility ("hidden"))) pml_start(HPAIO_RECORD *hpaio)
          bug("failed to open pml channel: %s %d\n", __FILE__, __LINE__);
          goto bugout;
       }
-      SendScanEvent(hpaio->deviceuri, 2000, "event");  /* hpssd message scan started */
+      SendScanEvent(hpaio->deviceuri, EVENT_START_SCAN_JOB); 
    }
    if (!oldStuff)
    {
@@ -765,10 +765,10 @@ int __attribute__ ((visibility ("hidden"))) pml_start(HPAIO_RECORD *hpaio)
       goto bugout; 
 
    /* Read SOP record and set image pipeline input traits. */
-   traits.iPixelsPerRow = letoh16(ps->BlackPixelsPerRow);
-   traits.iBitsPerPixel = letoh16(ps->BlackBitsPerPixel);
-   traits.lHorizDPI = letoh16(ps->BlackHorzDPI);
-   traits.lVertDPI = letoh16(ps->BlackVertDPI);
+   traits.iPixelsPerRow = le16toh(ps->BlackPixelsPerRow);
+   traits.iBitsPerPixel = le16toh(ps->BlackBitsPerPixel);
+   traits.lHorizDPI = le16toh(ps->BlackHorzDPI);
+   traits.lVertDPI = le16toh(ps->BlackVertDPI);
 
    /* Set up image-processing pipeline. */
    switch(ps->Code)
@@ -925,7 +925,7 @@ int __attribute__ ((visibility ("hidden"))) pml_read(HPAIO_RECORD *hpaio, SANE_B
       if (pd->ID == ID_RASTER_DATA)
       {
          /* Raster Record */
-         hpaio->RecordSize = letoh16(pd->Size);
+         hpaio->RecordSize = le16toh(pd->Size);
          hpaio->RecordIndex = 0;
       }
       else if (pd->ID == ID_END_PAGE)
@@ -1049,7 +1049,7 @@ int __attribute__ ((visibility ("hidden"))) pml_cancel(HPAIO_RECORD *hpaio)
    {
       hpmud_close_channel(hpaio->deviceid, hpaio->cmd_channelid);
       hpaio->cmd_channelid = -1;
-      SendScanEvent(hpaio->deviceuri, 2001, "event");  /* hpssd message scan done */
+      SendScanEvent(hpaio->deviceuri, EVENT_END_SCAN_JOB);
    }
 
    return OK;
