@@ -179,5 +179,24 @@
 SANE_Status  __attribute__ ((visibility ("hidden"))) SclSendCommand(int deviceid, int channelid, int cmd, int param);
 SANE_Status __attribute__ ((visibility ("hidden"))) SclInquire(int deviceid, int channelid, int cmd, int param, int *pValue, char *buffer, int maxlen);
 
+/*
+ * Phase 2 partial rewrite. des 9/26/07
+ */
+
+/* Note ESC = 0x1b = '\e' . */
+#define SCL_QUERY_DUPLEX_SUPPORTED "\e*s13500E"            /* 1 = device can support duplex */
+#define SCL_QUERY_DUPLEX_VERT_FLIP_SUPPORTED "\e*s13501E"  /* duplex can flip vertical (they all should) */
+#define SCL_QUERY_DUPLEX_HORZ_FLIP_SUPPORTED "\e*s13502E"  /* duplex can flip horizontal (they all should) */
+
+#define SCL_SET_DUPLEX "\e*d%dD13505R"                  /* 1 = duplex is requested, 0 otherwise */
+#define SCL_SET_DUPLEX_VERT_FLIP "\e*d%dV13506R"        /* 1 = vertical flip is requested, 0 otherwise */
+#define SCL_SET_DUPLEX_HORZ_FLIP "\e*d%dH13507R"        /* 1 = horizontal flip is requested, 0 otherwise */
+#define SCL_SET_DUPLEX_ORIENTATION "\e*a%d13507R"       /* 0 = no change, 1 = needs vert flip, 2 = vert and horz flip */
+
+struct hpaioScanner_s;
+
+SANE_Status __attribute__ ((visibility ("hidden"))) scl_send_cmd(struct hpaioScanner_s *hpaio, const char *buf, int size);
+SANE_Status __attribute__ ((visibility ("hidden"))) scl_query_int(struct hpaioScanner_s *hpaio, const char *buf, int size, int *result);
+
 #endif
 
