@@ -317,6 +317,7 @@ class PrintSettingsToolbox(QToolBox):
 
 
     def updateUi(self, cur_device, cur_printer):
+        #print "updateUi(%s, %s)" % (cur_device, cur_printer)
         self.cur_device = cur_device
         self.cur_printer = cur_printer
 
@@ -357,9 +358,9 @@ class PrintSettingsToolbox(QToolBox):
                     current = current_options.get('pageset', '')
                     self.addControlRow("pageset", self.__tr("Page Set"),
                         cups.PPD_UI_PICKONE, current,
-                        [('', self.__tr("All pages")),
+                        [('all', self.__tr("All pages")),
                          ('even', self.__tr("Even pages")),
-                         ('odd', self.__tr("Odd pages"))], '', job_option=True)
+                         ('odd', self.__tr("Odd pages"))], 'all', job_option=True)
 
                     self.job_options['pageset'] = current
 #                    if current == u'even':
@@ -911,15 +912,13 @@ class PrintSettingsToolbox(QToolBox):
 
             if value == default:
                 DefaultButton.setEnabled(False)
-
             self.connect(DefaultButton, SIGNAL("clicked()"), self.DefaultButton_clicked)
-            self.connect(OnRadioButton, SIGNAL("toggled(bool)"), self.BoolRadioButtons_clicked)
-
 
             if value:
                 OnRadioButton.setChecked(True)
             else:
                 OffRadioButton.setChecked(True)
+            self.connect(OnRadioButton, SIGNAL("toggled(bool)"), self.BoolRadioButtons_clicked)
 
             if read_only:
                 OnRadioButton.setEnabled(False)
@@ -945,7 +944,7 @@ class PrintSettingsToolbox(QToolBox):
 
             HBoxLayout.addWidget(ComboBox)
 
-            DefaultButton = DefaultPushButton(self.widget,"DefaultButton", self.group, option,
+            DefaultButton = DefaultPushButton(self.widget, "DefaultButton", self.group, option,
                 choices, default, ComboBox, typ, job_option)
 
             ComboBox.setDefaultPushbutton(DefaultButton)
@@ -1569,8 +1568,8 @@ class PrintSettingsToolbox(QToolBox):
             if choice is not None:
                 if not sender.job_option:
                     self.removePrinterOption(sender.option)
-                    index = sender.control.findText(text)
-                    sender.control.setCurrentIndex(index)
+                index = sender.control.findText(text)
+                sender.control.setCurrentIndex(index)
 
                 #self.linkPrintoutModeAndQuality(sender.option, choice) # TODO:
                 sender.control.setFocus(Qt.OtherFocusReason)
